//
//  O3D_Camera.m
//  Objective3D
//
//  Created by charlie on 5/28/09.
//  Copyright 2009 One More Muse. All rights reserved.
//	// (modelview matrix * camera rotation quaternion) * camera location matrix

#import "O3D_Camera.h"

@implementation O3D_Camera
@synthesize x, y, z, movementSpeed, rotationSpeed, rotation;

- (id) init {
	if(self = [super init]) {
		x = 0; y = 0; z = 1;
		movementSpeed = .1;
		rotationSpeed = .01;
		rotation = [[O3D_Quaternion alloc] init];
		location = [[O3D_Matrix alloc] initWithIdentity];
		location.data[12] = -x;
		location.data[13] = -y;
		location.data[14] = -z;
		
		perspective = [[O3D_Matrix alloc] initWithIdentity];
	}

	return self;
}

- (void) load {
	glMultMatrixf([rotation matrix].data);
	glMultMatrixf(location.data);
}

- (void) moveX:(float)ax Y:(float)ay Z:(float)az {
	x += ax;
	y += ay;
	z += az;
	location.data[12] = -x;
	location.data[13] = -y;
	location.data[14] = -z;
}

- (void) left:(float)amount {
	O3D_Vector3 *v = [rotation vectorX];
	[self moveX:-v.x * amount Y:-v.y * amount Z:v.z * amount];
}
- (void) left { [self left:movementSpeed]; }

- (void) right:(float)amount {
	O3D_Vector3 *v = [rotation vectorX];
	[self moveX:v.x * amount Y:-v.y * amount Z:-v.z * amount];
}
- (void) right { [self right:movementSpeed]; }

- (void) up:(float)amount {
	O3D_Vector3 *v = [rotation vectorY];
	[self moveX:-v.x * amount Y:v.y * amount Z:-v.z * amount];
}
- (void) up { [self up:movementSpeed]; }

- (void) down:(float)amount {
	O3D_Vector3 *v = [rotation vectorY];
	[self moveX:v.x * amount Y:-v.y * amount Z:v.z * amount];
}
- (void) down { [self down:movementSpeed]; }

- (void) forward:(float)amount {
	O3D_Vector3 *v = [rotation vectorZ];
	[self moveX:v.x * amount Y:v.y * amount Z:-v.z * amount];
}
- (void) forward { [self forward:movementSpeed]; }

- (void) backward:(float)amount {
	O3D_Vector3 *v = [rotation vectorZ];
	[self moveX:-v.x * amount Y:-v.y * amount Z:v.z * amount];
}

- (void) backward { [self backward:movementSpeed]; }

- (void) pan:(float)amount {
	O3D_Quaternion *rq = [O3D_Quaternion quaternionFromAxis:YAXIS angle:amount];
	[rq invert];
	[rotation multiplyByQuaternion:rq];
	//NSLog([[rotation vectorY] description]);
}
- (void) panLeft { [self pan:-rotationSpeed]; }
- (void) panRight { [self pan:rotationSpeed]; }

- (void) tilt:(float)amount {
	O3D_Quaternion *rq = [O3D_Quaternion quaternionFromAxis:[rotation vectorX] angle:amount];
	[rq invert];
	[rotation multiplyByQuaternion:rq];
	//NSLog([[rotation vectorY] description]);
}
- (void) tiltUp { [self tilt:rotationSpeed]; }
- (void) tiltDown { [self tilt:-rotationSpeed]; }

- (void) rotateByAngle:(float)angle onAxis:(O3D_Vector3 *)axis {
	[rotation multiplyByQuaternion:[O3D_Quaternion quaternionFromAxis:axis angle:angle]];
}

- (void) setPerspectiveWithFOV:(float)fov aspect:(float)aspect zNear:(float)zNear zFar:(float)zFar {
	float sine, cotangent, deltaZ;
	
	fov = (degreesToRadians(fov) / 2.0f);
	deltaZ = (zFar - zNear);
	sine = sin(fov);
	
	cotangent = (cos(fov) / sine);
	
	perspective.data[0] = (cotangent / aspect);
	perspective.data[5] = cotangent;
	perspective.data[10] = (-(zFar + zNear) / deltaZ);
	perspective.data[11] = -1.0f;
	perspective.data[14] = ((-2.0f * zNear * zFar) / deltaZ);
	perspective.data[15] = 0.0f;
}

- (void) applyPerspective { glMultMatrixf(perspective.data); }

- (void) dealloc {
	[rotation release];
	[perspective release];
	[location release];
	[super dealloc];
}

@end
