//
//  O3D_Cube.m
//  O3D
//
//  Created by charlie on 6/4/09.
//  Copyright 2009 One More Muse. All rights reserved.
//

#import "O3D_Cube.h"
#import "O3D_Quaternion.h"
#import "O3D_Vector4.h"
#import "O3D_Texture.h"

@implementation O3D_Cube
@synthesize tex, position, rotation, rotateBy, size;

- (id) initWithPosition:(O3D_Vector4 *)pos rotation:(O3D_Quaternion *)q texture:(O3D_Texture *)t {
	if(self = [super init]) {
		position = pos;
		[position retain];
		
		rotation = q;
		[rotation retain];
		
		rotateBy = [[O3D_Quaternion alloc] init];
		
		size = 0.1f;
		
		if(t != nil) tex = t;
	}
	
	return self;
}

- (void) draw {
	if(tex != nil) [tex bind];
	
	glPushMatrix();
	glMultMatrixf([O3D_Matrix matrixWithPositionVector4:position].data);
	glMultMatrixf([rotation matrix].data);
	
	glBegin(GL_QUADS);
	{
		glColor4f(0,0,0,1);
		// Front Face
		glNormal3f( 0.0f, 0.0f, 1.0f);					// Normal Pointing Towards Viewer
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-size, -size,  size);	// Point 1 (Front)
		glTexCoord2f(1, 0.0f); glVertex3f( size, -size,  size);	// Point 2 (Front)
		glTexCoord2f(1, 1); glVertex3f( size,  size,  size);	// Point 3 (Front)
		glTexCoord2f(0.0f, 1); glVertex3f(-size,  size,  size);	// Point 4 (Front)
		// Back Face
		glNormal3f( 0.0f, 0.0f,-1.0f);					// Normal Pointing Away From Viewer
		glTexCoord2f(1, 0.0f); glVertex3f(-size, -size, -size);	// Point 1 (Back)
		glTexCoord2f(1, 1); glVertex3f(-size,  size, -size);	// Point 2 (Back)
		glTexCoord2f(0.0f, 1); glVertex3f( size,  size, -size);	// Point 3 (Back)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( size, -size, -size);	// Point 4 (Back)
		// Top Face
		glNormal3f( 0.0f, 1.0f, 0.0f);					// Normal Pointing Up
		glTexCoord2f(0.0f, 1); glVertex3f(-size,  size, -size);	// Point 1 (Top)
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-size,  size,  size);	// Point 2 (Top)
		glTexCoord2f(1, 0.0f); glVertex3f( size,  size,  size);	// Point 3 (Top)
		glTexCoord2f(1, 1); glVertex3f( size,  size, -size);	// Point 4 (Top)
		// Bottom Face
		glNormal3f( 0.0f,-1.0f, 0.0f);					// Normal Pointing Down
		glTexCoord2f(1, 1); glVertex3f(-size, -size, -size);	// Point 1 (Bottom)
		glTexCoord2f(0.0f, 1); glVertex3f( size, -size, -size);	// Point 2 (Bottom)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( size, -size,  size);	// Point 3 (Bottom)
		glTexCoord2f(1, 0.0f); glVertex3f(-size, -size,  size);	// Point 4 (Bottom)
		// Right face
		glNormal3f( 1.0f, 0.0f, 0.0f);					// Normal Pointing Right
		glTexCoord2f(1, 0.0f); glVertex3f( size, -size, -size);	// Point 1 (Right)
		glTexCoord2f(1, 1); glVertex3f( size,  size, -size);	// Point 2 (Right)
		glTexCoord2f(0.0f, 1); glVertex3f( size,  size,  size);	// Point 3 (Right)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( size, -size,  size);	// Point 4 (Right)
		// Left Face
		glNormal3f(-1.0f, 0.0f, 0.0f);					// Normal Pointing Left
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-size, -size, -size);	// Point 1 (Left)
		glTexCoord2f(1, 0.0f); glVertex3f(-size, -size,  size);	// Point 2 (Left)
		glTexCoord2f(1, 1); glVertex3f(-size,  size,  size);	// Point 3 (Left)
		glTexCoord2f(0.0f, 1); glVertex3f(-size,  size, -size);	// Point 4 (Left)
	}
	glEnd();
	
	glPopMatrix();
}

- (void)dealloc {
	[rotateBy release];
	[position release];
	[rotation release];
	
	[super dealloc];
}

@end
