//
//  O3D_Light.h
//  O3D
//
//  Created by charlie on 6/1/09.
//  Copyright 2009 One More Muse. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <OpenGL/OpenGL.h>

@class O3D_Vector4;

/// Class for creating and manipulating lights
@interface O3D_Light : NSObject {
	/// Ambient light color (4 floats)
	float *ambient;
	/// Diffuse light color (4 floats)
	float *diffuse;
	/// OpenGL light number
	int lightNumber;
	/// Vector representing position
	O3D_Vector4 *position;
	/// Vector representing light direction
	O3D_Vector4 *direction;
	/// Light attenuation
	float attenuation;
}

@property (nonatomic) float *ambient,  *diffuse;
@property (nonatomic) int lightNumber;
@property (nonatomic) float attenuation;
@property (nonatomic, retain) O3D_Vector4 *position, *direction;

/// Initialize with position, ambient and diffuse colors, and an integer that will point to the OpenGL light memory
- (id) initWithPosition:(float[4])aPosition ambientColor:(float[4])ambientColor diffuseColor:(float[4])diffuseColor usingLightNumber:(int)num;

/// Convert integer into GL_LIGHT constant values
- (int) getLightConstant:(int)aLightNumber;
/// Load the light
- (void) load;
/// Move the light by the specified amounts; probably should be moveByX...
- (void) moveX:(float)x Y:(float)y Z:(float)z;

@end
