//
//  O3D_Light.m
//  O3D
//
//  Created by charlie on 6/1/09.
//  Copyright 2009 One More Muse. All rights reserved.
//

#import "O3D_Light.h"
#import "O3D_Vector4.h"

@implementation O3D_Light
@synthesize ambient, diffuse, position, direction, lightNumber, attenuation;

- (id)initWithPosition:(float[4])aPosition ambientColor:(float[4])ambientColor diffuseColor:(float[4])diffuseColor usingLightNumber:(int)number {
	if(self = [super init]) {
		position = [[O3D_Vector4 alloc] initWithX:aPosition[0] Y:aPosition[1] Z:aPosition[2] W:aPosition[3]];
		direction = [[O3D_Vector4 alloc] initWithX:0.0f Y:0.0f Z:1.0f W:1.0f];
		ambient = ambientColor;
		diffuse = diffuseColor;
		lightNumber = [self getLightConstant:number];
		attenuation = 0.0f;
		glEnable(lightNumber);
		[self load];
	}
	return self;
}

- (void) load {
	float values[4];											
	[position asFloats:values];
	
	glLightfv(lightNumber, GL_AMBIENT, ambient);				
	glLightfv(lightNumber, GL_DIFFUSE, diffuse);				
	glLightfv(lightNumber, GL_POSITION, values );				
	glLightf(lightNumber, GL_CONSTANT_ATTENUATION, attenuation);
}

- (void) moveX:(float)x Y:(float)y Z:(float)z {
	position.x += x;
	position.y += y;
	position.z += z;
	
	float values[4];											
	[position asFloats:values];
	glLightfv(lightNumber, GL_POSITION, values );
}

- (void) setPositionToX:(float)x Y:(float)y Z:(float)z {
	position.x = x;
	position.y = y;
	position.z = z;
	
	float values[4];											
	[position asFloats:values];
	glLightfv(lightNumber, GL_POSITION, values );
}

- (int) getLightConstant:(int)aLightNumber {
	switch(aLightNumber) {
		case 0: return 0x4000; break;
		case 1: return 0x4001; break;
		case 2: return 0x4002; break;
		case 3: return 0x4003; break;
		case 4: return 0x4004; break;
		case 5: return 0x4005; break;
		case 6: return 0x4006; break;
		case 7: return 0x4007; break;
	}
	
	return -1;
}

- (void) dealloc {
	[position release];
	[direction release];
	[super dealloc];
}
@end
