//
//  O3D_Matrix.m
//  Objective3D
//
//  Created by charlie on 5/27/09.
//  Copyright 2009 One More Muse. All rights reserved.
//

#import "O3D_Matrix.h"
#import "O3D_Vector3.h"
#import "O3D_Quaternion.h"
#import "O3D_Vector4.h"

@implementation O3D_Matrix
@synthesize data;

- (id) init {
	if(self = [super init]) {
		data = malloc(sizeof(float) * 16);
	}
	
	return self;
}

- (id) initWithMatrix:(O3D_Matrix *)matrix {
	O3D_Matrix *m = [[O3D_Matrix alloc] init];
	
	for(int i = 0; i < 16; i++) 
		m.data[i] = matrix.data[i];
	
	return m;
}

- (id) initWithIdentity {
	O3D_Matrix *m = [[O3D_Matrix alloc] init];
	
	for(int i = 0; i < 16; i++) 
		m.data[i] = (i % 5 == 0) ? 1.0f : 0.0f;
	
	return m;
}

- (id) initWithPositionVector4:(O3D_Vector4 *)vector {
	O3D_Matrix *m = [[O3D_Matrix alloc] initWithIdentity];
	
	m.data[12] = vector.x;
	m.data[13] = vector.y;
	m.data[14] = vector.z;
	
	return m;
}

+ (O3D_Matrix *) matrixWithPositionVector4:(O3D_Vector4 *)vector {
	O3D_Matrix *m = [[[O3D_Matrix alloc] initWithPositionVector4:vector] autorelease];
	return m;
}


- (void) transpose {
	O3D_Matrix *holder = [[O3D_Matrix alloc] initWithMatrix:self];
	
	self.data[0] =  holder.data[0];
	self.data[1] =  holder.data[4];
	self.data[2] =  holder.data[8];
	self.data[3] =  holder.data[12];
	self.data[4] =  holder.data[1];
	self.data[5] =  holder.data[5];
	self.data[6] =  holder.data[9];
	self.data[7] =  holder.data[13];
	self.data[8] =  holder.data[2];
	self.data[9] =  holder.data[6];
	self.data[10] = holder.data[10];
	self.data[11] = holder.data[14];
	self.data[12] = holder.data[3];
	self.data[13] = holder.data[7];
	self.data[14] = holder.data[11];
	self.data[15] = holder.data[15];
	
	[holder release];
}

- (void)multiplyByMatrix:(O3D_Matrix *)matrix {
	O3D_Matrix *holder = [[O3D_Matrix alloc] initWithMatrix:self];
	
	int cellIndex, cellRow, cellColumn, rowAndColumnIndex;
	
	for (cellIndex = 0; cellIndex < 16; cellIndex++) {
		self.data[cellIndex] = 0.0f;
		
		cellRow = cellIndex % 4;
		cellColumn = cellIndex / 4;
		
		for (rowAndColumnIndex = 0; rowAndColumnIndex < 4; rowAndColumnIndex++) {
			self.data[cellIndex] += holder.data[cellRow + (rowAndColumnIndex * 4)] * matrix.data[(cellColumn * 4) + rowAndColumnIndex];
		}
	}

	[holder release];
}

- (void)addMatrix:(O3D_Matrix *)matrix {
	O3D_Matrix *holder = [[O3D_Matrix alloc] initWithMatrix:self];
	
	for(int i = 0; i < 16; i++) 
		self.data[i] = holder.data[i] + matrix.data[i];
	
	[holder release];
}

- (void) rotateOnAxis:(O3D_Vector3 *)axis withAngle:(float)angle {
	O3D_Matrix *rotationMatrix;
	O3D_Quaternion *quat;
	
	quat = [[O3D_Quaternion alloc] initFromAxis:axis angle:angle];
	rotationMatrix = [quat matrix];
	
	[self multiplyByMatrix:rotationMatrix];
	
	[quat release];
}

- (void)dealloc {
	free(data);
	[super dealloc];
}

@end
