//
//  O3D_Quaternion.h
//  Objective3D
//
//  Created by charlie on 5/28/09.
//  Copyright 2009 One More Muse. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "O3D_Vector3.h"
#import "O3D_Matrix.h"

/// General purpose Quaternion class for camera and object rotations
@interface O3D_Quaternion : NSObject {
	float x, y, z, w;
}

@property (nonatomic) float x, y, z, w;

/// Creates a normalized Quaternion on the specified axis with the specified angle
- (id) initFromAxis:(O3D_Vector3 *)axis angle:(float)angle;
/// Copy a Quaternion
- (id) initWithQuaternion:(O3D_Quaternion *)aq;

/// Autoreleased Quaternion
+ (O3D_Quaternion *) quaternion;

/// Autoreleased copied Quaternion
+ (O3D_Quaternion *) quaternionWithQuaternion:(O3D_Quaternion *)aq;

/// Autoreleased Quaternion on the specified axis with the specified angle
+ (O3D_Quaternion *) quaternionFromAxis:(O3D_Vector3 *)axis angle:(float)angle;

/// Creates an autoreleased matrix representing the receiver
- (O3D_Matrix *) matrix;

/// generates a Dictionary cotaining an axis / angle pair for the receiver. Keys "axis" and "angle"
- (NSDictionary *) axisAngle;

/// Creates an autoreleased vector representing the X axis of the receiver
- (O3D_Vector3 *) vectorX;
/// Creates an autoreleased vector representing the Y axis of the receiver
- (O3D_Vector3 *) vectorY;
/// Creates an autoreleased vector representing the Z axis of the receiver
- (O3D_Vector3 *) vectorZ;

/// Inverts the receiver
- (void) invert;
/// Normalizes the receiver
- (void) normalize;
/// Multiplies the receiver by the argument Quaternion
- (void) multiplyByQuaternion:(O3D_Quaternion *)q2;
/// Rotates the receiver on the specified axis by the specified amount
- (void) rotateWithAxis:(O3D_Vector3 *)axis angle:(float)angle;

@end
