//
//  O3D_Quaternion.m
//  Objective3D
//
//  Created by charlie on 5/28/09.
//  Copyright 2009 One More Muse. All rights reserved.
//

#import "O3D_Quaternion.h"

@implementation O3D_Quaternion
@synthesize x, y, z, w;

- (id) init {
	if(self = [super init]) {
		x = 0; y = 0; z = 0; w = 1;
	}
	
	return self;
}

+ (O3D_Quaternion *) quaternion {
	O3D_Quaternion *q = [[[O3D_Quaternion alloc] init] autorelease];
	return q;
}

- (id) initWithQuaternion:(O3D_Quaternion *)aq {
	O3D_Quaternion *q = [[O3D_Quaternion alloc] init];
	
	q.x = aq.x;
	q.y = aq.y;
	q.z = aq.z;
	q.w = aq.w;
	
	return q;
}

+ (O3D_Quaternion *) quaternionWithQuaternion:(O3D_Quaternion *)aq  {
	O3D_Quaternion *q = [[[O3D_Quaternion alloc] initWithQuaternion:aq] autorelease];
	return q;
}

- (id) initFromAxis:(O3D_Vector3 *)axis angle:(float)angle {
	O3D_Quaternion *q = [[O3D_Quaternion alloc] init];
	float sinAngle;
	
	[axis normalize];
	
	angle *= 0.5f;
	sinAngle = sinf(angle);
	q.x = axis.x * sinAngle;
	q.y = axis.y * sinAngle;
	q.z = axis.z * sinAngle;
	q.w = cosf(angle);

	return q;
}

+ (O3D_Quaternion *) quaternionFromAxis:(O3D_Vector3 *)axis angle:(float)angle {
	O3D_Quaternion *q = [[[O3D_Quaternion alloc] initFromAxis:axis angle:angle] autorelease];
	return q;
}


- (NSDictionary *) axisAngle {
	NSDictionary *result;
	float sinAngle, angle;
	O3D_Vector3 *v;
	
	v = [[O3D_Vector3 alloc] init];
	O3D_Quaternion *q = [[O3D_Quaternion alloc] initWithQuaternion:self];
	[q normalize];
	
	sinAngle = sqrtf(1.0f - (q.w * q.w));
	if(fabs(sinAngle) < 0.0005f) sinAngle = 1.0f;
	
	v.x = q.x / sinAngle;
	v.y = q.y / sinAngle;
	v.z = q.z / sinAngle;
	
	angle = acos(q.w) * 2.0f;
	
	result = [NSDictionary dictionaryWithObjectsAndKeys:v, @"axis", [NSNumber numberWithFloat:angle], @"angle", nil]; 
	
	[q release];

	return result;
}

- (O3D_Matrix *) matrix {
	O3D_Matrix *matrix = [[[O3D_Matrix alloc] init] autorelease];
	
	matrix.data[0]  = (1.0f - (2.0f * ((y * y) + (z * z))));
	matrix.data[1]  =         (2.0f * ((x * y) + (z * w)));
	matrix.data[2]  =         (2.0f * ((x * z) - (y * w)));
	matrix.data[3]  = 0.0f;
	matrix.data[4]  =         (2.0f * ((x * y) - (z * w)));
	matrix.data[5]  = (1.0f - (2.0f * ((x * x) + (z * z))));
	matrix.data[6]  =         (2.0f * ((y * z) + (x * w)));
	matrix.data[7]  = 0.0f;
	matrix.data[8]  =         (2.0f * ((x * z) + (y * w)));
	matrix.data[9]  =         (2.0f * ((y * z) - (x * w)));
	matrix.data[10] = (1.0f - (2.0f * ((x * x) + (y * y))));
	matrix.data[11] = 0.0f;
	matrix.data[12] = 0.0f;
	matrix.data[13] = 0.0f;
	matrix.data[14] = 0.0f;
	matrix.data[15] = 1.0f;

	return matrix;
}

- (void)multiplyByQuaternion:(O3D_Quaternion *)q2 {
	O3D_Vector3 *vector1, *vector2, *cross;
	float angle;
	
	vector1 = [[O3D_Vector3 alloc] initWithX:x Y:y Z:z];
	vector2 = [[O3D_Vector3 alloc] initWithX:q2.x Y:q2.y Z:q2.z];
	
	angle = (w * q2.w) - [vector1 dotProductWithVector:vector2];
	cross = [vector1 crossProductWithVector:vector2];
	
	vector1.x *= q2.w;
	vector1.y *= q2.w;
	vector1.z *= q2.w;
	vector2.x *= w;
	vector2.y *= w;
	vector2.z *= w;
	
	x = vector1.x + vector2.x + cross.x;
	y = vector1.y + vector2.y + cross.y;
	z = vector1.z + vector2.z + cross.z;
	w = angle;
	
	[vector1 release];
	[vector2 release];
	[cross release];
}

- (void) invert {
	float length;
	length = 1 / ((x * x) + (y * y) + (z * z) + (w * w));
	
	x *= -length;
	y *= -length;
	z *= -length;
	w *= length;
}

- (void) rotateWithAxis:(O3D_Vector3 *)axis angle:(float)angle {
	O3D_Quaternion *q = [O3D_Quaternion quaternionFromAxis:axis angle:angle];
	[self multiplyByQuaternion:q];
}

- (void) normalize {
	float magnitude;
	
	magnitude = sqrt( (x * x) + (y * y) + (z * z) + (w * w) );
	x /= magnitude;
	y /= magnitude;
	z /= magnitude;
	w /= magnitude;
}

- (O3D_Vector3 *) vectorX {
	O3D_Vector3 *v = [O3D_Vector3 vector];
	v.x = 1.0 - 2.0*y*y - 2.0*z*z;
	v.y = 2.0*x*y + 2.0*z*w;
	v.z = 2.0*x*z - 2.0*y*w;
	return v;
}

- (O3D_Vector3 *) vectorY {
	O3D_Vector3 *v = [O3D_Vector3 vector];
	v.x = 2.0*x*y - 2.0*z*w;
	v.y = 1.0 - 2.0*x*x - 2.0*z*z;
	v.z = 2.0*y*z + 2.0*x*w;
	return v;
}

- (O3D_Vector3 *) vectorZ {
	O3D_Vector3 *v = [O3D_Vector3 vector];
	v.x = 2.0*x*z + 2.0*y*w;
	v.y = 2.0*y*z - 2.0*x*w;
	v.z = 1.0 - 2.0*x*x - 2.0*y*y;
	return v;
}

- (NSString *)description {
	return [NSString stringWithFormat:@"x :: %f, y :: %f, z :: %f, w :: %f", x, y, z, w];
}
@end
