//
//  O3D_Shader.h
//  O3D
//
//  Created by charlie on 5/31/09.
//  Copyright 2009 One More Muse. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <OpenGL/OpenGL.h>

/// Convenience wrapper for loading shaders and manipulating variables inside of them
@interface O3D_Shader : NSObject {
	/// The shader program
	GLhandleARB program;
	/// Vertex shader
	GLhandleARB vertex;
	/// Fragment shader
	GLhandleARB fragment;
}

@property (nonatomic) GLhandleARB program, vertex, fragment;

/// Initialize giving the paths to the vertex shader and fragment shader code. Either argument can be nil to only use one shader
- (id) initWithVertexPath:(NSString *)vertexPath fragmentPath:(NSString *)fragmentPath;

/// Change a variable representing one int inside the shader
- (void) setUniformInt:(int)value forVariableNamed:(char *)name;
/// Change a variable representing an array of ints inside the shader
- (void) setUniformInts:(int *)values forVariableNamed:(char *)name length:(int)length;
/// Change a variable representing one float inside the shader
- (void) setUniformFloat:(float)value forVariableNamed:(char *)name;
/// Change a variable representing an array of floats inside the shader
- (void) setUniformFloats:(float *)values forVariableNamed:(char *)name length:(int)length;
/// Change a variable representing one bool inside the shader
- (void) setUniformBool:(bool)value forVariableNamed:(char *)name;
/// Change a variable representing an array of floats inside the shader
- (void) setUniformBools:(bool *)values forVariableNamed:(char *)name length:(int)length;


@end
