//
//  O3D_Shader.m
//  O3D
//
//  Created by charlie on 5/31/09.
//  Copyright 2009 One More Muse. All rights reserved.
//

#import "O3D_Shader.h"


@implementation O3D_Shader
@synthesize program, vertex, fragment;

- (id) initWithVertexPath:(NSString *)vertexPath fragmentPath:(NSString *)fragmentPath {
	if(self = [super init]) {
		char err_str[1024];
		GLint status;
		
		BOOL vertexCompile = (vertexPath != nil) ? YES : NO;
		BOOL fragmentCompile = (fragmentPath != nil) ? YES : NO;
		
		program = glCreateProgramObjectARB();
		
		if(vertexCompile) {
			NSString * vertexSourceString = [NSString stringWithContentsOfFile:vertexPath encoding:1 error:nil];
			const char *vertexSource = [vertexSourceString UTF8String];
			
			vertex = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
			glShaderSourceARB(vertex, 1, &vertexSource, NULL);
			glCompileShaderARB(vertex);
			glGetShaderiv((GLuint)vertex, GL_COMPILE_STATUS, &status);
			
			if(status == GL_FALSE) {
				glGetShaderInfoLog((GLuint)vertex, sizeof(err_str), NULL, err_str);
				NSLog(@"\n\n\nvertex shader compile error: ");
				printf(err_str);
				return nil;
			} else {
				NSLog(@"vertex shader compiled OK!");
				glAttachObjectARB(program, vertex);
			}
		}	
		
		if(fragmentCompile) {
			NSString * fragmentSourceString = [NSString stringWithContentsOfFile:fragmentPath encoding:1 error:nil];
			const char *fragmentSource = [fragmentSourceString UTF8String];
			
			fragment = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
			glShaderSourceARB(fragment, 1, &fragmentSource, NULL);
			glCompileShaderARB(fragment);
			glGetShaderiv((GLuint)fragment, GL_COMPILE_STATUS, &status);
			
			if(status == GL_FALSE) {
				glGetShaderInfoLog((GLuint)fragment, sizeof(err_str), NULL, err_str);
				NSLog(@"\n\n\nfragment shader compile error: ");
				printf(err_str);
				return nil;
			} else {
				NSLog(@"fragment shader compiled OK!");
				glAttachObjectARB(program, fragment);
			}
		}
				
		glLinkProgramARB(program);
		glGetShaderiv((GLuint)program, GL_LINK_STATUS, &status);
		if(status == GL_FALSE) {
			glGetShaderInfoLog((GLuint)program, sizeof(err_str), NULL, err_str);
			NSLog(@"\n\n\nshader program compile error: ");
			printf(err_str);
			return nil;
		} else NSLog(@"shader program compiled OK!");
		
		glUseProgramObjectARB(program);
	}
	
	return self;
}

- (void) setUniformFloat:(float)value forVariableNamed:(char *)name {
	GLint loc = glGetUniformLocationARB(program, name);
	glUniform1fARB(loc, value);
}

- (void) setUniformFloats:(float *)values forVariableNamed:(char *)name length:(int)length {
	GLint loc = glGetUniformLocationARB(program, name);
	switch(length) {
		case 1:
			glUniform1fARB(loc, values[0]);
			break;
		case 2:
			glUniform2fARB(loc, values[0], values[1]);
			break;
		case 3:
			glUniform3fARB(loc, values[0], values[1], values[2]);
			break;
		case 4:
			glUniform4fARB(loc, values[0], values[1], values[2], values[4]);
			break;
	}
}

- (void) setUniformInt:(int)value forVariableNamed:(char *)name {
	GLint loc = glGetUniformLocationARB(program, name);
	glUniform1iARB(loc, value);
}

- (void) setUniformInts:(int *)values forVariableNamed:(char *)name length:(int)length {
	GLint loc = glGetUniformLocationARB(program, name);
	switch(length) {
		case 1:
			glUniform1iARB(loc, values[0]);
			break;
		case 2:
			glUniform2iARB(loc, values[0], values[1]);
			break;
		case 3:
			glUniform3iARB(loc, values[0], values[1], values[2]);
			break;
		case 4:
			glUniform4iARB(loc, values[0], values[1], values[2], values[4]);
			break;
	}
}

- (void) setUniformBool:(bool)value forVariableNamed:(char *)name {
	GLint loc = glGetUniformLocationARB(program, name);
	glUniform1iARB(loc, value);
}

- (void) setUniformBools:(bool *)values forVariableNamed:(char *)name length:(int)length {
	GLint loc = glGetUniformLocationARB(program, name);
	switch(length) {
		case 1:
			glUniform1iARB(loc, values[0]);
			break;
		case 2:
			glUniform2iARB(loc, values[0], values[1]);
			break;
		case 3:
			glUniform3iARB(loc, values[0], values[1], values[2]);
			break;
		case 4:
			glUniform4iARB(loc, values[0], values[1], values[2], values[4]);
			break;
	}
}

@end
