//
//  O3D_Texture.m
//  Objective3D
//
//  Code adapted from : http://mitchallen.com/iphone/archives/75
//

#import "O3D_Texture.h"

@implementation O3D_Texture
@synthesize texID;

- (id)initWithBitmap:(NSString *)filename {
	if(self = [super init]) {
		[self loadBitmap:filename intoIndex:0];
	}
	
	return self;
}

- (void) bind {
	glBindTexture( GL_TEXTURE_2D, texID );   // Select our texture
}

- (BOOL) loadBitmap:(NSString *)filename intoIndex:(int)texIndex {
	NSBitmapImageRep *rep = [NSBitmapImageRep imageRepWithContentsOfFile:filename];
	
	if(rep == nil) NSLog(@"ALERT");
	CGImageRef spriteImage = rep.CGImage;

	size_t w = CGImageGetWidth(spriteImage);
	size_t h = CGImageGetHeight(spriteImage);

	if(spriteImage)	{
		GLubyte *spriteData = (GLubyte *) malloc(w * h * 4);
		
		// FIX BELOW TO FLIP IMAGE TO MATCH OPENGL COORDINATES (flip y)
		/*NSLog(@"width : %d :: height: %d", w, h);
		 int bytesPRow, bytesPerPixel;
		 int rowNum, destRowNum;
		 bytesPerPixel = [rep bitsPerPixel] / 8;

		bytesPerPixel = [rep bitsPerPixel] / 8;
		GLubyte *spriteData = (GLubyte *) malloc(w * h * bytesPerPixel);
		GLubyte *holder = (GLubyte *)malloc(w * h * bytesPerPixel);
		char *theImageData = (char *)[ rep bitmapData ];

		memcpy(holder, theImageData, w * h * bytesPerPixel);
		
		bytesPRow = [ rep bytesPerRow ];
		destRowNum = 0;
		for( rowNum = h - 1; rowNum >= 0; rowNum--, destRowNum++ ){
			memcpy(theImageData + ( rowNum * bytesPerPixel * w ), holder + ( destRowNum * bytesPerPixel * w ), bytesPerPixel * w );
		}*/
		
		// Use the bitmap creation function provided by the Core Graphics framework.
		CGContextRef spriteContext = CGBitmapContextCreate(spriteData, w, h, 8, w * 4, CGImageGetColorSpace(spriteImage), kCGImageAlphaPremultipliedLast);
		
		// After we create the context, we can draw the sprite image to the context.
		CGContextDrawImage(spriteContext, CGRectMake(0.0, 0.0, (CGFloat)w, (CGFloat)h), spriteImage);
		
		// We don't need the context at this point, so we need to release it to avoid memory leaks.
		CGContextRelease(spriteContext);
		
		GLint saveName;
		
		// Now use OpenGL ES to generate a name for the texture. Pass by reference so that our texture variable gets set.
		glGenTextures(1, &texID);
		
		glGetIntegerv(GL_TEXTURE_BINDING_2D, &saveName);
		
		// Bind the texture name.
		glBindTexture(GL_TEXTURE_2D, texID);
		
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		
		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
		
		// Specify a 2D texture image, providing a pointer to the image data in memory
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, spriteData);
		
		glBindTexture(GL_TEXTURE_2D, saveName);
		
		free(spriteData);

	}
	return YES;
}

- (void) dealloc {
	[super dealloc];
}

@end
