//
//  O3D_Vector3.h
//  Objective3D
//
//  Created by charlie on 4/30/09.
//  Copyright 2009 One More Muse. All rights reserved.
//

#import <Cocoa/Cocoa.h>

/// Vector class representing three floats
@interface O3D_Vector3 : NSObject {
	// Data members
	float x, y, z;
}

@property (nonatomic) float x, y, z;

/// Initialize with X, Y and Z values
- (id) initWithX:(float)ax Y:(float)ay Z:(float)az;
/// Initialize with values from another vector
- (id) initWithVector:(O3D_Vector3 *)vector;
/// Initialize wiht an angle (Z assumed to be 0)
- (id) initWithAngle:(float)angle;

/// Create an autoreleased vector
+ (O3D_Vector3 *) vector;
/// Create an autoreleased vector with X, Y and Z values
+ (O3D_Vector3 *) vectorWithX:(float)ax Y:(float)ay Z:(float)az;
/// Create an autoreleased vector representing an nflw
+ (O3D_Vector3 *) vectorWithAngle:(float)angle;
/// Create an autoreleased vector with values from another vector
+ (O3D_Vector3 *) vectorWithVector:(id)vector;

/// Normalize the receiver
- (void) normalize;
/// Rotate the receiver using Quaternions
- (void) rotate:(float)rotation;
/// Get the angle of the vector
- (float) angle;
/// Get the dot product of the receiver and the argument vector
- (float) dotProductWithVector:(O3D_Vector3 *)vector;
/// Get the difference in angle between the receiver and the argument vector
- (float) angleWithVector:(O3D_Vector3 *)vector;

/// Create a normalized vector
- (O3D_Vector3 *) normalizedVector;
/// Create a vector with the specified rotation
- (O3D_Vector3 *) vectorWithRotation:(float)rotation;
/// Create a vector by calculating the cross product of the receiver and the argument vector
- (O3D_Vector3 *) crossProductWithVector:(O3D_Vector3 *)vector;

@end
