//
//  O3D_Vector3.m
//  Objective3D
//
//  Created by charlie on 4/30/09.
//  Copyright 2009 One More Muse. All rights reserved.
//

#import "O3D_Vector3.h"
#import "O3D_Quaternion.h"

@implementation O3D_Vector3
@synthesize x, y, z;

- (id) init {
	if(self = [super init]) {
		x = 0.0f; y = 0.0f; z = 0.0f;
	}
	return self;
}

+ (O3D_Vector3 *) vector {
	O3D_Vector3 *v = [[[O3D_Vector3 alloc] init] autorelease];
	return v;
}

- (id) initWithX:(float)ax Y:(float)ay Z:(float)az {
	if(self = [self init]) {
		x = ax; y = ay; z = az;
	}
	return self;
}

+ (O3D_Vector3 *) vectorWithX:(float)ax Y:(float)ay Z:(float)az {
	O3D_Vector3 *v = [[[O3D_Vector3 alloc] initWithX:ax Y:ay Z:az] autorelease];
	return v;
}

- (id) initWithAngle:(float)angle {
	float angleInRads = (angle / 360) * (M_PI * 2);
		
	float _x = cosf(angleInRads);
	float _y = sinf(angleInRads);
	
	return [self initWithX:_x Y:_y Z:0.0f];
}

+ (O3D_Vector3 *) vectorWithAngle:(float)angle {
	O3D_Vector3 *v = [[[O3D_Vector3 alloc] initWithAngle:angle] autorelease];
	return v;
}

- (id) initWithVector:(O3D_Vector3 *)vector {
	if(self = [self init]) {
		x = vector.x;
		y = vector.y;
		z = vector.z;
	}
	return self;
}

+ (O3D_Vector3 *) vectorWithVector:(id)vector {
	O3D_Vector3 *v = [[[O3D_Vector3 alloc] initWithVector:vector] autorelease];
	return v;
}

- (void)normalize {
	float magnitude;
	
	magnitude = sqrt( (x * x) + (y * y) + (z * z) );
					 
	x /= magnitude;
	y /= magnitude;
	z /= magnitude;
}

- (O3D_Vector3 *) normalizedVector {
	O3D_Vector3 *v = [[O3D_Vector3 alloc] initWithVector:self];
	[v normalize];
	return v;
}

- (O3D_Vector3 *) vectorWithRotation:(float)rotation {
	O3D_Vector3 *v = [[O3D_Vector3 alloc] initWithVector:self];
	[v rotate:rotation];
	return v;
}

- (O3D_Vector3 *) crossProductWithVector:(O3D_Vector3 *)vector {
	O3D_Vector3 *result = [[O3D_Vector3 alloc] init];
	
	result.x = ( (y * vector.z) - (z * vector.y) );
	result.y = ( (z * vector.x) - (x * vector.z) );
	result.z = ( (x * vector.y) - (y * vector.x) );
	
	return result;
}

- (float) angle {
	return atan2(y, x);
}

- (void) rotate:(float)angle {
	O3D_Quaternion *rotationQuaternion, *vectorQuaternion, *inverseQuaternion;	
	rotationQuaternion = [O3D_Quaternion quaternionFromAxis:self angle:angle];
	
	vectorQuaternion = [O3D_Quaternion quaternion];
	vectorQuaternion.x = x;
	vectorQuaternion.y = y;
	vectorQuaternion.z = z;
	vectorQuaternion.w = 0.0f;
	
	inverseQuaternion = [O3D_Quaternion quaternionWithQuaternion:rotationQuaternion];
	[inverseQuaternion invert];
	
	[vectorQuaternion multiplyByQuaternion:inverseQuaternion];
	[vectorQuaternion multiplyByQuaternion:rotationQuaternion];
	
	x = vectorQuaternion.x;
	y = vectorQuaternion.y;
	z = vectorQuaternion.z;
	
}

- (float) dotProductWithVector:(O3D_Vector3 *)vector {
	return ( (x * vector.x) + (y * vector.y) + (z * vector.z) );
}

- (float) angleWithVector:(O3D_Vector3 *)vector {
	float dp = [self dotProductWithVector:vector];
	return acosf(dp);
}

- (NSString *)description {
	return [NSString stringWithFormat:@"x :: %f, y :: %f, z :: %f", x, y, z];
}
@end
