//
//  O3D_Vector4.m
//  O3D
//
//  Created by charlie on 6/2/09.
//  Copyright 2009 One More Muse. All rights reserved.
//

#import "O3D_Vector4.h"
#import "O3D_Vector3.h"
#import "O3D_Quaternion.h"


@implementation O3D_Vector4
@synthesize x, y, z, w;

- (id) init {
	if(self = [super init]) {
		x = 0.0f; y = 0.0f; z = 0.0f; w = 1.0f;
	}
	return self;
}

+ (O3D_Vector4 *) vector {
	O3D_Vector4 *v = [[[O3D_Vector4 alloc] init] autorelease];
	return v;
}

- (id) initWithX:(float)ax Y:(float)ay Z:(float)az W:(float)aw {
	if(self = [self init]) {
		x = ax; y = ay; z = az; w = aw;
	}
	return self;
}

+ (O3D_Vector4 *) vectorWithX:(float)ax Y:(float)ay Z:(float)az W:(float)aw {
	O3D_Vector4 *v = [[[O3D_Vector4 alloc] initWithX:ax Y:ay Z:az W:aw] autorelease];
	return v;
}

- (id) initWithAngle:(float)angle {
	float angleInRads = (angle / 360) * (M_PI * 2);
	
	float _x = cosf(angleInRads);
	float _y = sinf(angleInRads);
	
	return [self initWithX:_x Y:_y Z:0.0f W:1.0f];
}

+ (O3D_Vector4 *) vectorWithAngle:(float)angle {
	O3D_Vector4 *v = [[[O3D_Vector4 alloc] initWithAngle:angle] autorelease];
	return v;
}

- (id) initWithVector:(id)vector {
	if(self = [self init]) {
		x = ((O3D_Vector4 *)vector).x;
		y = ((O3D_Vector4 *)vector).y;
		z = ((O3D_Vector4 *)vector).z;
		w = ([vector isMemberOfClass:[O3D_Vector4 class]]) ? ((O3D_Vector4 *)vector).w : 1.0f;
	}
	return self;
}

+ (O3D_Vector4 *) vectorWithVector:(id)vector {
	O3D_Vector4 *v = [[[O3D_Vector4 alloc] initWithVector:vector] autorelease];
	return v;
}

- (void)normalize {
	float magnitude;
	
	magnitude = sqrt( (x * x) + (y * y) + (z * z) + (w * w));
	
	x /= magnitude;
	y /= magnitude;
	z /= magnitude;
	w /= magnitude;
}

- (O3D_Vector4 *) normalizedVector {
	O3D_Vector4 *v = [[[O3D_Vector4 alloc] initWithVector:self] autorelease];
	[v normalize];
	return v;
}

- (O3D_Vector4 *) vectorWithRotation:(float)rotation {
	O3D_Vector4 *v = [[[O3D_Vector4 alloc] initWithVector:self] autorelease];
	[v rotate:rotation];
	return v;
}

- (void) rotate:(float)angle {
	O3D_Quaternion *rotationQuaternion, *vectorQuaternion, *inverseQuaternion;	
	rotationQuaternion = [[O3D_Quaternion alloc] initFromAxis:[[O3D_Vector3 alloc] initWithX:0.0 Y:1.0f Z:0.0f] angle:angle];
	
	vectorQuaternion.x = x;
	vectorQuaternion.y = y;
	vectorQuaternion.z = z;
	vectorQuaternion.w = 1.0f;
	
	inverseQuaternion = [[O3D_Quaternion alloc] initWithQuaternion:rotationQuaternion];
	[inverseQuaternion invert];
	
	[vectorQuaternion multiplyByQuaternion:inverseQuaternion];
	[rotationQuaternion multiplyByQuaternion:vectorQuaternion];
	
	x = rotationQuaternion.x;
	y = rotationQuaternion.y;
	z = rotationQuaternion.z;	
}

- (float) dotProductWithVector:(O3D_Vector4 *)vector {
	return ( (x * vector.x) + (y * vector.y) + (z * vector.z) + (w * vector.w) );
}

- (float) angleWithVector:(O3D_Vector4 *)vector {
	float dp = [self dotProductWithVector:vector];
	return acosf(dp);
}

- (float) angle {
	return atan2(y, x);
}

- (void) asFloats:(float *)values {
	values[0] = x; values[1] = y; values[2] = z; values[3] = w;
}

- (O3D_Vector3 *) vector3 {
	return [[O3D_Vector3 alloc] initWithX:x Y:y Z:z];
}



@end
